#!/usr/bin/env python3

# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

# pylint: skip-file
# flake8: noqa
# type: ignore

# default_value1 = 1
# default_value2 = 2
# default_value3 = 3
# default_value4 = 4

import sys

# get a subpackage from package
from foo import bar

# get a module
from foo.bar import bif as bif_mod

# get bif function from bar
from foo.bar import bif_func as bif

# get the baz function imported by bar, but for which there is also a baz module in the foo/bar directory
from foo.bar import baz

import os

# get function and variable from foo but defined elsewhere
from foo import zed
from foo import foo_name

# FIXME: optionally embed modules and packages local to the script or current working directory
import localpkg
import localpkg.bar
import localmod

print(f"{foo_name=}")
print(f"{zed()=}")
print(f"{bar.baz()=}")
print(f"{bar.bif_func()=}")
print(f"{bar.zed_func()=}")
print(f"{baz()=}")
print(f"{bif()=}")
print(f"{bif_mod.bif()=}")
print(f"{bif_mod.zed()=}")

mods = (
    "foo",
    "foo.bar",
    "foo.bar.baz",
    "foo.bar.bif",
    "foo.bar.zed",
    "localpkg",
    "localpkg.bar",
    "localpkg.bar.baz",
    "localmod",
    "sys",
)
for mod_name in mods:
    try:
        mod = sys.modules[mod_name]
        try:
            mod__name__ = mod.__name__
        except AttributeError:
            mod__name__ = None
        try:
            mod__package__ = mod.__package__
        except AttributeError:
            mod__package__ = None
        try:
            mod__file__ = mod.__file__
        except AttributeError:
            mod__file__ = None
        print(f"MODULE {mod_name}: {mod__name__=}, {mod__package__=}, {mod__file__=}, {repr(mod)=}, {id(mod)=}")
        if mod__file__:
            del sys.modules[mod_name]
    except KeyError:
        print(f"MODULE {mod_name}: NOT FOUND in sys.modules")
