# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import pytest

from Octeres.utils.debugging import wrap_and_detect_modification
from Octeres.util import superprint


def test_wrap_and_detect_modification_00():
    def func(a, b, c=""):
        return f"a:{a},b:{b},c:{c}"

    f = wrap_and_detect_modification(func)
    f(["a", "b"], 2, c="string")


def test_wrap_and_detect_modification_01():
    def func(a: list, b, c=""):
        a.append("modified!")
        return f"a:{a},b:{b},c:{c}"

    f = wrap_and_detect_modification(func)
    with pytest.raises(AssertionError):
        f(["a", "b"], 2, c="string")


def test_wrap_and_detect_modification_02():
    class A:
        def func_a(self, a: list, b, c=""):
            return f"a:{a},b:{b},c:{c}"

        def func_b(self, a: list, b, c=""):
            a.append("modified!")
            return f"a:{a},b:{b},c:{c}"

    for name, value in A.__dict__.items():
        if hasattr(value, "__call__"):
            func = wrap_and_detect_modification(value)
            setattr(A, name, func)
            assert getattr(A, name) == func
    obj = A()
    obj.func_a(["a", "b"], 2, c="string")
    with pytest.raises(AssertionError):
        obj.func_b(["a", "b"], 2, c="string")


def test_wrap_and_detect_modification_03():
    try:
        import numpy as np
    except ImportError:
        pass
    else:
        superprint("Numpy found, executing 03")
        a = np.zeros(10)
        b = np.zeros(10)

        def func(a: np.ndarray, b, c=""):
            a[0] = 1
            return f"a:{a},b:{b},c:{c}"

        f = wrap_and_detect_modification(func)
        with pytest.raises(AssertionError):
            f(a, b, c="string")
